unit uNewName;
{$INCLUDE 'NewName.inc'}

interface

uses
  Windows, SysUtils, Classes,
  {$IFDEF FAR3} Plugin3, {$ELSE} PluginW, {$ENDIF} FarKeysW, FarColor,
  uLog, uFAR;

procedure SetStartupInfoW(const Info: TPluginStartupInfo); stdcall;
procedure ExitFARW( {$IFDEF FAR3} const Info: TExitInfo {$ENDIF} ); stdcall;
{$IFDEF FAR3}
procedure GetGlobalInfoW(var Info: TGlobalInfo); stdcall;
function OpenW(var Info: TOpenInfo): THandle; stdcall;
{$ELSE}
function OpenPluginW(OpenFrom: integer; Item: integer): THandle; stdcall;
{$ENDIF}
procedure GetPluginInfoW(var Info: TPluginInfo); stdcall;

implementation

uses
  uSystem, uNewNameConsts, uNewNameDialog, uMessages, uOptions, uFiles;

const
  PLUGIN_TITLE = 'NewName';
  PLUGIN_DESCRIPTION = 'New Name';
  PLUGIN_AUTHOR = 'Pepak, http://www.pepak.net';

procedure Replace(var Str: string; const Replacement: string; var Index: integer; RemoveChars: integer);
var
  i, n: integer;
begin
  n := Length(Replacement);
  // If I need to replace more characters than I have available, I add a dummy string
  if n > RemoveChars then begin
    System.Insert(StringOfChar('.', n-RemoveChars), Str, Index);
    RemoveChars := n;
  end;
  // Perform the actual replacement
  for i := 1 to n do begin
    Str[Index] := Replacement[i];
    Inc(Index);
  end;
  // If some extra characters are left from the original string, delete them
  if n < RemoveChars then
    System.Delete(Str, Index, RemoveChars-n);
end;

function LeadingZeroes(const Str: string; Options: TNewNameOptions): string;
var
  n: integer;
begin
  if Options.NoOddZeroes then
    Result := Str
  else begin
    n := Options.StartingNumberLength - Length(Str);
    if n > 0 then
      Result := StringOfChar('0', n) + Str
    else
      Result := Str;
  end;
end;

function ApplyFileNameTemplate(const FileName: string; Options: TNewNameOptions; Counter, NumberOfFiles: integer): string;
var
  i, n: Integer;
  C: Char;
begin
  Result := Options.FileNamePattern;
  i := 1;
  n := Length(Result);
  while i < n do begin
    if Result[i] <> '%' then
      Inc(i)
    else begin
      C := Result[Succ(i)];
      case C of
        '#':      // Ordinal number of file
          Replace(Result, LeadingZeroes(IntToStr(Counter), Options), i, 2);
        'h':      // Ordinal number of file, hex, lower-case
          Replace(Result, LeadingZeroes(LowerCase(Format('%x', [Counter])), Options), i, 2);
        'H':      // Ordinal number of file, hex, upper-case
          Replace(Result, LeadingZeroes(UpperCase(Format('%x', [Counter])), Options), i, 2);
        't', 'T': // Total number of files
          Replace(Result, LeadingZeroes(IntToStr(NumberOfFiles), Options), i, 2);
        'f', 'F': // Filename and extension
          Replace(Result, FileName, i, 2);
        'n', 'N': // Filename without extension
          Replace(Result, ChangeFileExt(FileName, ''), i, 2);
        'e', 'E': // Extension (without the dot)
          Replace(Result, Copy(ExtractFileExt(FileName), 2, MaxInt), i, 2);
        else      // As-is
          Replace(Result, C, i, 2);
      end;
      n := Length(Result);
    end;
  end;
end;

function DoRenameFiles(Panel: TFarPanelInfo; Options: TNewNameOptions): boolean;
var
  Dir, SourceFN, DestFN, FailuresFN: string;
  i, NumberOfFiles, Counter: integer;
  FailuresStream: TTextStream;
begin
  Result := False;
  NumberOfFiles := 0;
  for i := 0 to Pred(Panel.Items.Count) do
    if Panel.Items[i].Selected then
      Inc(NumberOfFiles);
  if NumberOfFiles > 0 then begin
    Result := True;
    Counter := Options.StartingNumber;
    FailuresFN := '';
    FailuresStream := nil;
    try
      for i := 0 to Pred(Panel.Items.Count) do begin
        if Panel.Items[i].Selected then begin
          Dir := TFarUtils.FullFileName(IncludeTrailingPathDelimiter(Panel.Directory), True);
          SourceFN := Panel.Items[i].FileName;
          DestFN := ApplyFileNameTemplate(SourceFN, Options, Counter, NumberOfFiles);
          Inc(Counter);
          //Log('Renaming "%s" to "%s"', [SourceFN, DestFN]);
          if (SourceFN = DestFn) or RenameFile(Dir + SourceFN, Dir + DestFN) then
            Panel.Items[i].Selected := False
          else begin
            Result := False;
            if FailuresStream = nil then begin
              FailuresFN := SystemTempFile(SystemTempDir, 'newname', '.txt');
              FailuresStream := TTextStream.Create(FailuresFN, fmCreate);
              FailuresStream.Encoding := {$IFDEF UNICODE} feUtf8 {$ELSE} feAnsi {$ENDIF} ;
              FailuresStream.WriteLine(GetMsg(MFailuresInfo));
            end;
            FailuresStream.WriteLine(AnsiQuotedStr(SourceFN, '"') + ' -> ' + AnsiQuotedStr(DestFN, '"'));
          end;
        end;
      end;
      if (FailuresStream <> nil) and FileExists(FailuresFN) then begin
        FreeAndNil(FailuresStream);
        FarApi.Editor(PFarChar(FailuresFN), GetMsg(MTitle), 0, 0, -1, -1,
          EF_NONMODAL or EF_IMMEDIATERETURN or EF_DELETEONLYFILEONCLOSE or EF_DISABLEHISTORY {$IFDEF FAR3} or EF_DISABLESAVEPOS {$ENDIF} ,
          0, 1, {$IFDEF UNICODE} CP_UTF8 {$ELSE} CP_ANSI {$ENDIF}
          );
      end;
    finally
      FreeAndNil(FailuresStream);
    end;
  end;
end;

procedure SetStartupInfoW(const Info: TPluginStartupInfo);
begin
  FarApi := Info;
  FSF := Info.FSF^;
  FarApi.FSF := @FSF;
end;

procedure ExitFARW( {$IFDEF FAR3} const Info: TExitInfo {$ENDIF} );
begin
end;

{$IFDEF FAR3}
procedure GetGlobalInfoW(var Info: TGlobalInfo);
var
  FN: string;
  VerHigh, VerLow, VerBuild: LongWord;
begin
  FillChar(Info, Sizeof(Info), 0);
  Info.StructSize := Sizeof(Info);
  Info.MinFarVersion := MakeFarVersion( {$IFDEF FAR3} 3, 0, 0, 0, 0 {$ELSE} 2, 0, 1807 {$ENDIF} );
  FN := GetModuleName(HInstance);
  if GetFileVersion(FN, VerHigh, VerLow, VerBuild) then
    Info.Version := MakeFarVersion( {$IFDEF FAR3} VerHigh, VerLow, 0, VerBuild, 0 {$ELSE} VerHigh, VerLow, VerBuild {$ENDIF} )
  else
    Info.Version := MakeFarVersion( {$IFDEF FAR3} 0, 0, 0, 0, 0 {$ELSE} 0, 0 ,0 {$ENDIF} );
  Info.Guid := PLUGIN_GUID;
  Info.Title := PLUGIN_TITLE;
  Info.Description := PLUGIN_DESCRIPTION;
  Info.Author := PLUGIN_AUTHOR;
end;
{$ENDIF}

var
  PluginInfoInitialized: boolean = False;
  PanelMenuStrings:  array[0..0] of PFarChar;
  PanelMenuGUIDs:    array[0..0] of TGUID;
  //ConfigMenuStrings: array[0..0] of PFarChar;
  //ConfigMenuGUIDs:   array[0..0] of TGUID;

procedure GetPluginInfoW(var Info: TPluginInfo);
begin
  // Initialize messages and GUIDs
  if not PluginInfoInitialized then begin
    PanelMenuStrings[0] := GetMsg(MTitle);
    PanelMenuGUIDs[0] := MENU_PANEL_GUID;
    //ConfigMenuStrings[0] := GetMsg(MTitle);
    //ConfigMenuGUIDs[0] := MENU_CONFIG_GUID;
    PluginInfoInitialized := True;
  end;
  FillChar(Info, Sizeof(Info), 0);
  Info.StructSize := Sizeof(Info);
  Info.Flags := PF_EDITOR;
  {$IFDEF FAR3}
  Info.PluginMenu.Count := Length(PanelMenuStrings);
  Info.PluginMenu.Strings := @PanelMenuStrings[0];
  Info.PluginMenu.Guids := @PanelMenuGUIDs[0];
  //Info.PluginConfig.Count := Length(ConfigMenuStrings);
  //Info.PluginConfig.Strings := @ConfigMenuStrings[0];
  //Info.PluginConfig.Guids := @ConfigMenuGUIDs[0];
  {$ELSE}
  Info.PluginMenuStringsNumber := Length(PanelMenuStrings);
  Info.PluginMenuStrings := @PanelMenuStrings[0];
  //Info.PluginConfigStringsNumber := Length(ConfigMenuStrings);
  //Info.PluginConfigStrings := @ConfigMenuStrings[0];
  {$ENDIF}
end;

{$IFDEF FAR3}
function OpenW(var Info: TOpenInfo): THandle;
{$ELSE}
function OpenPluginW(OpenFrom: integer; Item: integer): THandle;
{$ENDIF}
var
  Panel: TFarPanelInfo;
  Options: TNewNameOptions;
  Dlg: TNewNameDialog;
begin
  Result := 0;
  Panel := TFarPanelInfo.Create(True);
  try
    if Panel.SelectedItemsCount <= 0 then
      FarMessage([MTitle, MNoFilesSelected, MOk], FMSG_WARNING, 1)
    else begin
      Options := TNewNameOptions.Create;
      try
        Options.Load;
        Dlg := TNewNameDialog.Create;
        try
          Dlg.Options := Options;
          if Dlg.Execute then begin
            Options.Save;
            DoRenameFiles(Panel, Options);
            //Panel.Refresh;
            Panel.ApplySelection;
          end;
        finally
          FreeAndNil(Dlg);
        end;
      finally
        FreeAndNil(Options);
      end;
    end;
  finally
    FreeAndNil(Panel);
  end;
end;

exports
  SetStartupInfoW,
  ExitFARW,
  {$IFDEF FAR3}
  GetGlobalInfoW,
  OpenW,
  {$ELSE}
  OpenPluginW,
  {$ENDIF}
  GetPluginInfoW;

initialization
  {$IFDEF FAR3}
  uFar.PluginID := PLUGIN_GUID;
  {$ENDIF}

finalization

end.
