unit uNewNameDialog;
{$INCLUDE 'NewName.inc'}

interface

uses
  Windows, SysUtils, Classes,
  {$IFDEF FAR3} Plugin3, {$ELSE} PluginW, {$ENDIF} FarKeysW, FarColor,
  uLog, uFAR, uNewNameConsts, uOptions, uMessages;

type
  TNewNameDialog = class(TFarDialog)
  private
    fOptions: TNewNameOptions;
    fFileNamePatternBuf: array[0..MAX_PATH] of TFarChar;
    fStartNumberBuf: array[0..15] of TFarChar;
  private
    fEditFileNamePattern: integer;
    fEditStartNumber: integer;
    fCheckNoOddZeroes: integer;
    fButtonOK: integer;
    fButtonCancel: integer;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure ClearDialog; override;
    function Execute: boolean;
    property Options: TNewNameOptions read fOptions write fOptions;
  end;

implementation

{ TNewNameDialog }

procedure TNewNameDialog.ClearDialog;
begin
  inherited;
  fEditFileNamePattern := -1;
  fEditStartNumber := -1;
  fCheckNoOddZeroes := -1;
  fButtonOK := -1;
  fButtonCancel := -1;
end;

constructor TNewNameDialog.Create;
begin
  inherited Create( {$IFDEF FAR3} PLUGIN_GUID, DIALOG_MAIN_GUID {$ELSE} FarApi.ModuleNumber {$ENDIF} );
end;

destructor TNewNameDialog.Destroy;
begin
  inherited;
end;

function TNewNameDialog.Execute: boolean;
var
  i, Button: integer;
  NumStr: string;
  NumOK: boolean;
begin
  Result := False;
  // Set up the dialog variables
  StrPCopy(fFileNamePatternBuf, Copy(Options.FileNamePattern, 1, Length(fFileNamePatternBuf)));
  StrPCopy(fStartNumberBuf, Copy(Options.StartingNumberAsString, 1, Length(fStartNumberBuf)));
  // Build the dialog
  ClearDialog;
                          AddDoubleBox(  3,  1, 56, 10, GetMsg(MTitle), 0);
                          AddLabel    (  5,  2,         GetMsg(MEnterBeginningOfFileName), 0);
  fEditFileNamePattern := AddEdit     (  5,  3, 49,     @fFileNamePatternBuf[0], nil, 'NewNamePattern', Pred(Length(fFileNamePatternBuf)), DIF_HISTORY or DIF_NOAUTOCOMPLETE);
  fEditStartNumber :=     AddEdit     (  5,  4,  6,     @fStartNumberBuf[0], nil, nil, Pred(Length(fStartNumberBuf)), DIF_MASKEDIT or DIF_NOAUTOCOMPLETE);
                          AddLabel    ( 12,  4,         GetMsg(MStartFromThisNumber), 0);
                          AddLabel    (  0,  5,         nil, DIF_SEPARATOR);
  fCheckNoOddZeroes :=    AddCheckbox (  5,  6,         GetMsg(MNoOddZeroes), Options.NoOddZeroes, 0);
                          // Empty line
                          AddLabel    (  0,  8,         nil, DIF_SEPARATOR);
  fButtonOK :=            AddButton   (  0,  9,         GetMsg(MOk), DIF_CENTERGROUP);
  fButtonCancel :=        AddButton   (  0,  9,         GetMsg(MCancel), DIF_CENTERGROUP);
  // Set the focused control
  {$IFDEF FAR3}
  Items[fEditFileNamePattern].Flags := Items[fEditFileNamePattern].Flags or DIF_FOCUS;
  {$ELSE}
  Items[fEditFileNamePattern].Focus := 1;
  {$ENDIF}
  // Set the default button
  {$IFDEF FAR3}
  Items[fButtonOK].Flags := Items[fButtonOK].Flags or DIF_DEFAULTBUTTON;
  {$ELSE}
  Items[fButtonOK].DefaultButton := 1;
  {$ENDIF}
  // Show the dialog
  if Build(-1, -1, 60, 12, 0, 'Contents') then begin
    if inherited Execute(Button) then begin
      if Button = fButtonOK then begin
        NumStr := Trim(ItemText[fEditStartNumber]);
        NumOK := True;
        if NumStr = '' then
          NumStr := '0'
        else
          for i := 1 to Length(NumStr) do
            if not CharInSet(NumStr[i], ['0'..'9']) then begin
              NumOK := False;
              FarMessage([MTitle, MInvalidNumber, MOk], FMSG_WARNING, 1);
              Break;
            end;
        if NumOK then begin
          //Log('OK pressed, reading the settings');
          Options.FileNamePattern := Trim(ItemText[fEditFileNamePattern]);
          Options.StartingNumberAsString := NumStr;
          Options.NoOddZeroes := ItemChecked[fCheckNoOddZeroes];
          Result := True;
          //Log('Pattern="%s"', [Options.FileNamePattern]);
          //Log('StartingNumberAsString="%s"', [Options.StartingNumberAsString]);
          //Log('StartingNumber=%d', [Options.StartingNumber]);
          //Log('StartingNumberLength=%d', [Options.StartingNumberLength]);
          //Log('NoOddZeroes=%d', [Integer(Options.NoOddZeroes)]);
        end;
      end;
    end;
  end;
end;

end.
